from .result_query import QueryData
from .result_query import QueryDataCatchment
from .result_query import QueryDataGlobal
from .result_query import QueryDataNode
from .result_query import QueryDataReach
from .result_query import QueryDataStructure
