from .result_catchment import ResultCatchment
from .result_catchments import ResultCatchments
from .result_gridpoint import ResultGridPoint
from .result_location import ResultLocation
from .result_locations import ResultLocations
from .result_network import ResultNetwork
from .result_node import ResultNode
from .result_nodes import ResultNodes
from .result_quantity import ResultQuantity
from .result_reach import ResultReach
from .result_reaches import ResultReaches
from .various import make_proper_variable_name
